@@ -1,17 +1,8 @@
-%% The contents of this file are subject to the Mozilla Public License
-%% Version 1.1 (the "License"); you may not use this file except in
-%% compliance with the License. You may obtain a copy of the License at
-%% http://www.mozilla.org/MPL/
+%% This Source Code Form is subject to the terms of the Mozilla Public
+%% License, v. 2.0. If a copy of the MPL was not distributed with this
+%% file, You can obtain one at https://mozilla.org/MPL/2.0/.
 %%
-%% Software distributed under the License is distributed on an "AS IS"
-%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
-%% License for the specific language governing rights and limitations
-%% under the License.
-%%
-%% The Original Code is RabbitMQ.
-%%
-%% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2020 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2020 VMware, Inc. or its affiliates.  All rights reserved.
 %%
 
 -module(amqp10_client).
@@ -19,11 +10,6 @@
 -include("amqp10_client.hrl").
 -include_lib("amqp10_common/include/amqp10_framing.hrl").
 
--ifdef(nowarn_deprecated_gen_fsm).
--compile({nowarn_deprecated_function,
-          [{gen_fsm, send_event, 2}]}).
--endif.
-
 -export([open_connection/1,
          open_connection/2,
          close_connection/1,
@@ -167,7 +153,7 @@ begin_session_sync(Connection, Timeout) when is_pid(Connection) ->
 %% {amqp10_event, {session, SessionPid, {ended, Why}}}
 -spec end_session(pid()) -> ok.
 end_session(Pid) ->
-    gen_fsm:send_event(Pid, 'end').
+    amqp10_client_session:'end'(Pid).
 
 %% @doc Synchronously attach a link on 'Session'.
 %% This is a convenience function that awaits attached event
