@@ -24,6 +24,10 @@
 -define(TRACE_SINK, '__trace_sink').
 -define(ROTATE_TIMEOUT, 100000).
 
+-ifdef(TEST).
+-include_lib("eunit/include/eunit.hrl").
+-endif.
+
 %% API
 -export([start/0,
         log/3, log/4, log/5,
@@ -31,7 +35,8 @@
         md/0, md/1,
         rotate_handler/1, rotate_handler/2, rotate_sink/1, rotate_all/0,
         trace/2, trace/3, trace_file/2, trace_file/3, trace_file/4, trace_console/1, trace_console/2,
-        list_all_sinks/0, clear_all_traces/0, stop_trace/1, stop_trace/3, status/0,
+        install_trace/2, install_trace/3, remove_trace/1, trace_state/3, trace_func/3,
+        list_all_sinks/0, clear_all_traces/0, clear_trace_by_destination/1, stop_trace/1, stop_trace/3, status/0,
         get_loglevel/1, get_loglevel/2, set_loglevel/2, set_loglevel/3, set_loglevel/4, get_loglevels/1,
         update_loglevel_config/1, posix_error/1, set_loghwm/2, set_loghwm/3, set_loghwm/4,
         safe_format/3, safe_format_chop/3, unsafe_format/2, dispatch_log/5, dispatch_log/7, dispatch_log/9,
@@ -42,8 +47,31 @@
 
 -export_type([log_level/0, log_level_number/0]).
 
+-record(trace_func_state_v1, {
+          pid :: undefined | pid(),
+          level :: log_level(),
+          count :: infinity | pos_integer(),
+          format_string :: string(),
+          timeout :: infinity | pos_integer(),
+          started = os:timestamp() :: erlang:timestamp() %% use os:timestamp for compatability
+         }).
+
 %% API
 
+%% @doc installs a lager trace handler into the target process (using sys:install) at the specified level.
+-spec install_trace(pid(), log_level()) -> ok.
+install_trace(Pid, Level) ->
+    install_trace(Pid, Level, []).
+
+-spec install_trace(pid(), log_level(), [{count, infinity | pos_integer()} | {format_string, string()} | {timeout, timeout()}]) -> ok.
+install_trace(Pid, Level, Options) ->
+    sys:install(Pid, {fun ?MODULE:trace_func/3, trace_state(Pid, Level, Options)}).
+
+%% @doc remove a previously installed lager trace handler from the target process.
+-spec remove_trace(pid()) -> ok.
+remove_trace(Pid) ->
+    sys:remove(Pid, fun ?MODULE:trace_func/3).
+
 %% @doc Start the application. Mainly useful for using `-s lager' as a command
 %% line switch to the VM to make lager start on boot.
 start() -> start(lager).
@@ -325,6 +353,12 @@ clear_traces_by_sink(Sinks) ->
                   end,
                   Sinks).
 
+clear_trace_by_destination(ID) ->
+    Sinks = lists:sort(list_all_sinks()),
+    Traces = find_traces(Sinks),
+    [ stop_trace_int({Filter, Level, Destination}, Sink) || {Sink, {Filter, Level, Destination}} <- Traces, Destination == ID].
+
+
 clear_all_traces() ->
     Handlers = lager_config:global_get(handlers, []),
     clear_traces_by_sink(list_all_sinks()),
@@ -359,32 +393,25 @@ status() ->
     end,
     Status = ["Lager status:\n",
         [begin
-                    Level = get_loglevel(Sink, Handler),
-                    case Handler of
-                        {lager_file_backend, File} ->
-                            io_lib:format("File ~s (~s) at level ~p\n", [File, Sink, Level]);
-                        lager_console_backend ->
-                            io_lib:format("Console (~s) at level ~p\n", [Sink, Level]);
-                        _ ->
-                            []
-                    end
-            end || {Handler, _Watcher, Sink} <- lists:sort(fun({_, _, S1},
-                                                               {_, _, S2}) -> S1 =< S2 end,
-                                                           Handlers)],
+             Level = get_loglevel(Sink, Handler),
+             get_sink_handler_status(Sink, Handler, Level)
+         end || {Handler, _Watcher, Sink} <- lists:sort(fun({_, _, S1},
+                                                            {_, _, S2}) -> S1 =< S2 end,
+                                                        Handlers)],
         "Active Traces:\n",
         [begin
-                    LevelName = case Level of
-                        {mask, Mask} ->
-                            case lager_util:mask_to_levels(Mask) of
-                                [] -> none;
-                                Levels -> hd(Levels)
-                            end;
-                        Num ->
-                            lager_util:num_to_level(Num)
-                    end,
-                    io_lib:format("Tracing messages matching ~p (sink ~s) at level ~p to ~p\n",
-                        [Filter, Sink, LevelName, Destination])
-            end || {Sink, {Filter, Level, Destination}} <- Traces],
+             LevelName = case Level of
+                 {mask, Mask} ->
+                     case lager_util:mask_to_levels(Mask) of
+                         [] -> none;
+                         Levels -> hd(Levels)
+                     end;
+                 Num ->
+                     lager_util:num_to_level(Num)
+             end,
+             io_lib:format("Tracing messages matching ~p (sink ~s) at level ~p to ~p\n",
+                           [Filter, Sink, LevelName, Destination])
+         end || {Sink, {Filter, Level, Destination}} <- Traces],
          [
          "Tracing Reductions:\n",
             case ?DEFAULT_TRACER:info('query') of
@@ -402,6 +429,15 @@ status() ->
          ]],
     io:put_chars(Status).
 
+get_sink_handler_status(Sink, Handler, Level) ->
+    case Handler of
+        {lager_file_backend, File} ->
+            io_lib:format("File ~ts (~s) at level ~p\n", [File, Sink, Level]);
+        lager_console_backend ->
+            io_lib:format("Console (~s) at level ~p\n", [Sink, Level]);
+        _ ->
+            []
+    end.
 
 %% @doc Set the loglevel for a particular backend.
 set_loglevel(Handler, Level) when is_atom(Level) ->
@@ -560,8 +596,8 @@ pr(Record, Module, Options) when is_tuple(Record), is_atom(element(1, Record)),
         error:undef ->
             Record
     end;
-pr(List, Module, Options) when is_list(List), is_list(Options) ->
-    [pr(Element, Module, Options) || Element <- List];
+pr([Head|Tail], Module, Options) when is_list(Options) ->
+    [pr(Head, Module, Options)|pr(Tail, Module, Options)];
 pr(Record, _, _) ->
     Record.
 
@@ -658,3 +694,53 @@ rotate_handler(Handler) ->
 
 rotate_handler(Handler, Sink) ->
     gen_event:call(Sink, Handler, rotate, ?ROTATE_TIMEOUT).
+
+%% @private
+trace_func(#trace_func_state_v1{pid=Pid, level=Level, format_string=Fmt}=FuncState, Event, ProcState) ->
+    _ = lager:log(Level, Pid, Fmt, [Event, ProcState]),
+    check_timeout(decrement_count(FuncState)).
+
+%% @private
+trace_state(Pid, Level, Options) ->
+    #trace_func_state_v1{pid=Pid,
+                         level=Level,
+                         count=proplists:get_value(count, Options, infinity),
+                         timeout=proplists:get_value(timeout, Options, infinity),
+                         format_string=proplists:get_value(format_string, Options, "TRACE ~p ~p")}.
+
+decrement_count(#trace_func_state_v1{count=infinity} = FuncState) ->
+    FuncState;
+decrement_count(#trace_func_state_v1{count=1}) ->
+    %% hit the counter limit
+    done;
+decrement_count(#trace_func_state_v1{count=Count} = FuncState) ->
+    FuncState#trace_func_state_v1{count=Count - 1}.
+
+check_timeout(#trace_func_state_v1{timeout=infinity} = FuncState) ->
+    FuncState;
+check_timeout(#trace_func_state_v1{timeout=Timeout, started=Started} = FuncState) ->
+    case (timer:now_diff(os:timestamp(), Started) / 1000) > Timeout of
+        true ->
+            done;
+        false ->
+            FuncState
+    end.
+
+-ifdef(TEST).
+get_sink_handler_status_ascii_test() ->
+    File = "C:\\ProgramData\\Directory With Spaces\\lager.log",
+    validate_status(File).
+
+get_sink_handler_status_latin_test() ->
+    File = "C:\\ProgramData\\Tést Directory\\lager.log",
+    validate_status(File).
+
+get_sink_handler_status_unicode_test() ->
+    File = "C:\\ProgramData\\찦차를 타고 온 펲시맨과 쑛다리 똠방각하 (Korean)\\lager.log",
+    validate_status(File).
+
+validate_status(File) ->
+    Handler = {lager_file_backend, File},
+    Status = get_sink_handler_status(?DEFAULT_SINK, Handler, debug),
+    ?assertNotEqual(nomatch, string:find(Status, File)).
+-endif.
