@@ -1,3 +1,9 @@
+%% This Source Code Form is subject to the terms of the Mozilla Public
+%% License, v. 2.0. If a copy of the MPL was not distributed with this
+%% file, You can obtain one at https://mozilla.org/MPL/2.0/.
+%%
+%% Copyright (c) 2017-2020 VMware, Inc. or its affiliates.  All rights reserved.
+%%
 %% @doc The primary module for interacting with ra servers and clusters.
 
 -module(ra).
@@ -38,6 +44,7 @@
          start_server/1,
          start_server/4,
          restart_server/1,
+         restart_server/2,
          stop_server/1,
          force_delete_server/1,
          trigger_election/1,
@@ -128,7 +135,27 @@ start_in(DataDir) ->
 -spec restart_server(ra_server_id()) -> ok | {error, term()}.
 restart_server(ServerId) ->
     % don't match on return value in case it is already running
-    case catch ra_server_sup_sup:restart_server(ServerId) of
+    case catch ra_server_sup_sup:restart_server(ServerId, #{}) of
+        {ok, _} -> ok;
+        {ok, _, _} -> ok;
+        {error, _} = Err -> Err;
+        {'EXIT', Err} -> {error, Err}
+    end.
+
+%% @doc Restarts a previously successfully started ra server
+%% @param ServerId the ra_server_id() of the server
+%% @param AddConfig additional config parameters to be merged into the
+%% original config.
+%% @returns `{ok | error, Error}' where error can be
+%% `not_found' or `name_not_registered' when the ra server has never before
+%% been started on the Erlang node.
+%% @end
+
+-spec restart_server(ra_server_id(), ra_server:mutable_config()) ->
+    ok | {error, term()}.
+restart_server(ServerId, AddConfig) ->
+    % don't match on return value in case it is already running
+    case catch ra_server_sup_sup:restart_server(ServerId, AddConfig) of
         {ok, _} -> ok;
         {ok, _, _} -> ok;
         {error, _} = Err -> Err;
@@ -210,10 +237,11 @@ start_or_restart_cluster(ClusterName, Machine, ServerIds) ->
     {error, cluster_not_formed}.
 start_or_restart_cluster(ClusterName, Machine,
                          [FirstServer | RemServers] = ServerIds, Timeout) ->
-    case ra_server_sup_sup:restart_server(FirstServer) of
+    case ra_server_sup_sup:restart_server(FirstServer, #{}) of
         {ok, _} ->
             %% restart the rest of the servers
-            _ = [{ok, _} = ra_server_sup_sup:restart_server(N) || N <- RemServers],
+            _ = [{ok, _} = ra_server_sup_sup:restart_server(N, #{})
+                 || N <- RemServers],
             {ok, ServerIds, []};
         {error, Err} ->
             ?ERR("start_or_restart_cluster: got an error: ~p~n", [Err]),
@@ -402,11 +430,11 @@ start_server(Conf) ->
             {error, invalid_uid}
     end.
 
-%% @doc Deletes a ra cluster in an orderly fashion
-%% This function commits and end of life command which after each server applies
-%% it will cause that server to shut down and delete all it's data.
+%% @doc Deletes a ra cluster in an orderly fashion.
+%% This function commits an end of life command which after each server applies
+%% it will cause that server to shut down and delete all its data.
 %% The leader will stay up until it has successfully replicated the end of life
-%% command to all servers after which it too will shut down and delete all it's
+%% command to all servers after which it too will shut down and delete all of its
 %% data.
 %% @param ServerIds the ra_server_ids of the cluster
 %% @returns `{{ok, Leader} | error, nodedown}'
@@ -768,7 +796,7 @@ pipeline_command(ServerId, Command) ->
 %% @end
 -spec local_query(ServerId :: ra_server_id(),
                   QueryFun :: query_fun()) ->
-    ra_server_proc:ra_leader_call_ret({ra_idxterm(), term()}).
+    ra_server_proc:ra_leader_call_ret({ra_idxterm(), Reply :: term()}).
 local_query(ServerId, QueryFun) ->
     local_query(ServerId, QueryFun, ?DEFAULT_TIMEOUT).
 
@@ -781,8 +809,8 @@ local_query(ServerId, QueryFun) ->
 -spec local_query(ServerId :: ra_server_id(),
                   QueryFun :: query_fun(),
                   Timeout :: timeout()) ->
-    ra_server_proc:ra_leader_call_ret({ra_idxterm(), term()}) |
-    {ok, {ra_idxterm(), term()}, not_known}.
+    ra_server_proc:ra_leader_call_ret({ra_idxterm(), Reply :: term()}) |
+    {ok, {ra_idxterm(), Reply :: term()}, not_known}.
 local_query(ServerId, QueryFun, Timeout) ->
     ra_server_proc:query(ServerId, QueryFun, local, Timeout).
 
@@ -798,8 +826,8 @@ local_query(ServerId, QueryFun, Timeout) ->
 %% @end
 -spec leader_query(ServerId :: ra_server_id() | [ra_server_id()],
                    QueryFun :: query_fun()) ->
-    ra_server_proc:ra_leader_call_ret(term()) |
-    {ok, {ra_idxterm(), term()}, not_known}.
+    ra_server_proc:ra_leader_call_ret({ra_idxterm(), Reply :: term()}) |
+    {ok, {ra_idxterm(), Reply :: term()}, not_known}.
 leader_query(ServerId, QueryFun) ->
     leader_query(ServerId, QueryFun, ?DEFAULT_TIMEOUT).
 
@@ -812,8 +840,8 @@ leader_query(ServerId, QueryFun) ->
 -spec leader_query(ServerId :: ra_server_id() | [ra_server_id()],
                    QueryFun :: query_fun(),
                    Timeout :: timeout()) ->
-    ra_server_proc:ra_leader_call_ret(term()) |
-    {ok, {ra_idxterm(), term()}, not_known}.
+    ra_server_proc:ra_leader_call_ret({ra_idxterm(), Reply :: term()}) |
+    {ok, {ra_idxterm(), Reply :: term()}, not_known}.
 leader_query(ServerId, QueryFun, Timeout) ->
     ra_server_proc:query(ServerId, QueryFun, leader, Timeout).
 
@@ -828,7 +856,7 @@ leader_query(ServerId, QueryFun, Timeout) ->
 %% @end
 -spec consistent_query(ServerId :: ra_server_id() | [ra_server_id()],
                        QueryFun :: query_fun()) ->
-    ra_server_proc:ra_leader_call_ret(term()).
+    ra_server_proc:ra_leader_call_ret(Reply :: term()).
 consistent_query(ServerId, QueryFun) ->
     consistent_query(ServerId, QueryFun, ?DEFAULT_TIMEOUT).
 
@@ -841,7 +869,7 @@ consistent_query(ServerId, QueryFun) ->
 -spec consistent_query(ServerId :: ra_server_id() | [ra_server_id()],
                        QueryFun :: query_fun(),
                        Timeout :: timeout()) ->
-    ra_server_proc:ra_leader_call_ret(term()).
+    ra_server_proc:ra_leader_call_ret(Reply :: term()).
 consistent_query(ServerId, QueryFun, Timeout) ->
     ra_server_proc:query(ServerId, QueryFun, consistent, Timeout).
 
