@@ -1,3 +1,9 @@
+%% This Source Code Form is subject to the terms of the Mozilla Public
+%% License, v. 2.0. If a copy of the MPL was not distributed with this
+%% file, You can obtain one at https://mozilla.org/MPL/2.0/.
+%%
+%% Copyright (c) 2017-2020 VMware, Inc. or its affiliates.  All rights reserved.
+%%
 %% @hidden
 -module(ra_server_proc).
 
@@ -120,7 +126,8 @@
                ra_event_formatter :: undefined | {module(), atom(), [term()]},
                flush_commands_size = ?FLUSH_COMMANDS_SIZE :: non_neg_integer(),
                snapshot_chunk_size = ?DEFAULT_SNAPSHOT_CHUNK_SIZE :: non_neg_integer(),
-               receive_snapshot_timeout = ?DEFAULT_RECEIVE_SNAPSHOT_TIMEOUT :: non_neg_integer()
+               receive_snapshot_timeout = ?DEFAULT_RECEIVE_SNAPSHOT_TIMEOUT :: non_neg_integer(),
+               aten_poll_interval = 1000 :: non_neg_integer()
               }).
 
 -record(state, {conf :: #conf{},
@@ -160,8 +167,9 @@ cast_command(ServerId, Priority, Cmd) ->
 
 -spec query(server_loc(), query_fun(),
             local | consistent | leader, timeout()) ->
-    ra_server_proc:ra_leader_call_ret(term())
-    | {ok, Reply :: term(), not_known}.
+    ra_server_proc:ra_leader_call_ret({ra_idxterm(), Reply :: term()})
+    | ra_server_proc:ra_leader_call_ret(Reply :: term())
+    | {ok, {ra_idxterm(), Reply :: term()}, not_known}.
 query(ServerLoc, QueryFun, local, Timeout) ->
     statem_call(ServerLoc, {local_query, QueryFun}, Timeout);
 query(ServerLoc, QueryFun, leader, Timeout) ->
@@ -263,9 +271,10 @@ init(Config0 = #{id := Id, cluster_name := ClusterName}) ->
     FlushCommandsSize = application:get_env(ra, low_priority_commands_flush_size,
                                             ?FLUSH_COMMANDS_SIZE),
     SnapshotChunkSize = application:get_env(ra, snapshot_chunk_size,
-                                    ?DEFAULT_SNAPSHOT_CHUNK_SIZE),
+                                            ?DEFAULT_SNAPSHOT_CHUNK_SIZE),
     ReceiveSnapshotTimeout = application:get_env(ra, receive_snapshot_timeout,
                                                  ?DEFAULT_RECEIVE_SNAPSHOT_TIMEOUT),
+    AtenPollInt = application:get_env(aten, poll_interval, 1000),
     State = #state{conf = #conf{log_id = LogId,
                                 cluster_name = ClusterName,
                                 name = Key,
@@ -274,7 +283,8 @@ init(Config0 = #{id := Id, cluster_name := ClusterName}) ->
                                 ra_event_formatter = RaEventFormatterMFA,
                                 flush_commands_size = FlushCommandsSize,
                                 snapshot_chunk_size = SnapshotChunkSize,
-                                receive_snapshot_timeout = ReceiveSnapshotTimeout},
+                                receive_snapshot_timeout = ReceiveSnapshotTimeout,
+                                aten_poll_interval = AtenPollInt},
                    server_state = ServerState},
     ok = net_kernel:monitor_nodes(true, [nodedown_reason]),
     {ok, recover, State, [{next_event, cast, go}]}.
@@ -813,7 +823,7 @@ handle_event(_EventType, EventContent, StateName, State) ->
     {next_state, StateName, State}.
 
 terminate(Reason, StateName,
-          #state{conf = #conf{name = Key},
+          #state{conf = #conf{name = Key, cluster_name = ClusterName},
                  server_state = ServerState} = State) ->
     ?INFO("~s: terminating with ~w in state ~w~n",
           [log_id(State), Reason, StateName]),
@@ -822,6 +832,7 @@ terminate(Reason, StateName,
     Parent = ra_directory:where_is_parent(UId),
     case Reason of
         {shutdown, delete} ->
+            catch ra_leaderboard:clear(ClusterName),
             catch ra_directory:unregister_name(UId),
             catch ra_log_meta:delete_sync(UId),
             catch ets:delete(ra_state, UId),
@@ -883,13 +894,13 @@ handle_enter(RaftState, OldRaftState,
         true ->
             %% ensure transitions from and to leader are logged at a higher
             %% level
-            ?NOTICE("~s: ~s -> ~s in term: ~b~n",
+            ?NOTICE("~s: ~s -> ~s in term: ~b machine version: ~b~n",
                     [log_id(State), OldRaftState, RaftState,
-                     current_term(State)]);
+                     current_term(State), machine_version(State)]);
         false ->
-            ?DEBUG("~s: ~s -> ~s in term: ~b~n",
+            ?DEBUG("~s: ~s -> ~s in term: ~b machine version: ~b~n",
                    [log_id(State), OldRaftState, RaftState,
-                    current_term(State)])
+                    current_term(State), machine_version(State)])
     end,
     handle_effects(RaftState, Effects, cast,
                    State#state{server_state = ServerState}).
@@ -1203,6 +1214,9 @@ leader_id(#state{server_state = ServerState}) ->
 current_term(#state{server_state = ServerState}) ->
     ra_server:current_term(ServerState).
 
+machine_version(#state{server_state = ServerState}) ->
+    ra_server:machine_version(ServerState).
+
 process_pending_queries(NewLeader,
                         #state{server_state = ServerState0} = State) ->
     {ServerState, Froms} = ra_server:process_new_leader_queries(ServerState0),
@@ -1218,9 +1232,12 @@ election_timeout_action(really_short, #conf{broadcast_time = Timeout}) ->
 election_timeout_action(short, #conf{broadcast_time = Timeout}) ->
     T = rand:uniform(Timeout * ?DEFAULT_ELECTION_MULT) + Timeout,
     {state_timeout, T, election_timeout};
-election_timeout_action(long, #conf{broadcast_time = Timeout}) ->
-    %% this should be longer than aten detection poll interval
-    T = rand:uniform(Timeout * ?DEFAULT_ELECTION_MULT * 2) + 1000,
+election_timeout_action(long, #conf{broadcast_time = Timeout,
+                                    aten_poll_interval = Poll}) ->
+    %% this should be longer than aten detection poll interval so that
+    %% there is a chance a false positive from aten can be reversed without
+    %% triggering elections
+    T = rand:uniform(Timeout * ?DEFAULT_ELECTION_MULT * 2) + Poll,
     {state_timeout, T, election_timeout}.
 
 % sets the tick timer for periodic actions such as sending rpcs to servers
