@@ -1,11 +1,26 @@
+%% This Source Code Form is subject to the terms of the Mozilla Public
+%% License, v. 2.0. If a copy of the MPL was not distributed with this
+%% file, You can obtain one at https://mozilla.org/MPL/2.0/.
+%%
+%% Copyright (c) 2017-2020 VMware, Inc. or its affiliates.  All rights reserved.
+%%
 %% @hidden
 -module(ra_server_sup_sup).
 
 -behaviour(supervisor).
 
+-define(MUTABLE_CONFIG_KEYS,
+        [cluster_name,
+         metrics_key,
+         broadcast_time,
+         tick_timeout,
+         await_condition_timeout,
+         max_pipeline_count,
+         ra_event_formatter]).
+
 %% API functions
 -export([start_server/1,
-         restart_server/1,
+         restart_server/2,
          stop_server/1,
          delete_server/1,
          remove_all/0,
@@ -33,10 +48,14 @@ start_server(#{id := NodeId,
             Err
     end.
 
--spec restart_server(ra_server_id()) -> supervisor:startchild_ret().
-restart_server({RaName, Node}) ->
+-spec restart_server(ra_server_id(), ra_server:mutable_config()) ->
+    supervisor:startchild_ret().
+restart_server({RaName, Node}, AddConfig) ->
     case rpc:call(Node, ?MODULE, prepare_restart_rpc, [RaName]) of
-        {ok, Config} ->
+        {ok, Config0} ->
+            %% only certain config keys are mutable
+            Config = maps:merge(Config0,
+                                maps:with(?MUTABLE_CONFIG_KEYS, AddConfig)),
             supervisor:start_child({?MODULE, Node}, [Config]);
         {error, _} = Err ->
             Err;
@@ -129,29 +148,29 @@ delete_server_rpc(RaName) ->
 
 delete_data_directory(Directory) ->
     DeleteFunction = fun() ->
-                         try ra_lib:recursive_delete(Directory) of
-                            ok ->
-                                % moving on
-                                ok
-                         catch
-                            _:_ = Err ->
-                                ?WARN("delete_server/1 failed to delete directory ~s~n"
-                                    "Error: ~p~n", [Directory, Err]),
-                                error
-                         end
+                             try ra_lib:recursive_delete(Directory) of
+                                 ok ->
+                                     % moving on
+                                     ok
+                             catch
+                                 _:_ = Err ->
+                                     ?WARN("ra: delete_server/1 failed to delete directory ~s~n"
+                                           "Error: ~p~n", [Directory, Err]),
+                                     error
+                             end
                      end,
     case DeleteFunction() of
         ok ->
             ok;
         _ ->
             spawn(fun() ->
-                      ra_lib:retry(DeleteFunction, 2)
+                          ra_lib:retry(DeleteFunction, 2)
                   end)
     end.
 
 remove_all() ->
     _ = [begin
-             ?INFO("terminating child ~w~n", [Pid]),
+             ?DEBUG("ra: terminating child ~w~n", [Pid]),
              supervisor:terminate_child(?MODULE, Pid)
          end
          || {_, Pid, _, _} <- supervisor:which_children(?MODULE)],
