@@ -1,17 +1,8 @@
-%% The contents of this file are subject to the Mozilla Public License
-%% Version 1.1 (the "License"); you may not use this file except in
-%% compliance with the License. You may obtain a copy of the License
-%% at https://www.mozilla.org/MPL/
+%% This Source Code Form is subject to the terms of the Mozilla Public
+%% License, v. 2.0. If a copy of the MPL was not distributed with this
+%% file, You can obtain one at https://mozilla.org/MPL/2.0/.
 %%
-%% Software distributed under the License is distributed on an "AS IS"
-%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
-%% the License for the specific language governing rights and
-%% limitations under the License.
-%%
-%% The Original Code is RabbitMQ.
-%%
-%% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2020 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2020 VMware, Inc. or its affiliates.  All rights reserved.
 %%
 
 -module(rabbit_connection_tracking).
@@ -293,7 +284,7 @@ unregister_connection(ConnId = {Node, _Name}) when Node =:= node() ->
 -spec lookup(rabbit_types:connection_name()) -> rabbit_types:tracked_connection() | 'not_found'.
 
 lookup(Name) ->
-    Nodes = rabbit_mnesia:cluster_nodes(running),
+    Nodes = rabbit_nodes:all_running(),
     lookup(Name, Nodes).
 
 lookup(_, []) ->
@@ -312,7 +303,7 @@ list() ->
       fun (Node, Acc) ->
               Tab = tracked_connection_table_name_for(Node),
               Acc ++ mnesia:dirty_match_object(Tab, #tracked_connection{_ = '_'})
-      end, [], rabbit_mnesia:cluster_nodes(running)).
+      end, [], rabbit_nodes:all_running()).
 
 -spec count() -> non_neg_integer().
 
@@ -321,7 +312,7 @@ count() ->
       fun (Node, Acc) ->
               Tab = tracked_connection_table_name_for(Node),
               Acc + mnesia:table_info(Tab, size)
-      end, 0, rabbit_mnesia:cluster_nodes(running)).
+      end, 0, rabbit_nodes:all_running()).
 
 -spec list(rabbit_types:vhost()) -> [rabbit_types:tracked_connection()].
 
@@ -330,7 +321,7 @@ list(VHost) ->
       fun (Node, Acc) ->
               Tab = tracked_connection_table_name_for(Node),
               Acc ++ mnesia:dirty_match_object(Tab, #tracked_connection{vhost = VHost, _ = '_'})
-      end, [], rabbit_mnesia:cluster_nodes(running)).
+      end, [], rabbit_nodes:all_running()).
 
 
 -spec list_on_node(node()) -> [rabbit_types:tracked_connection()].
@@ -361,7 +352,7 @@ list_of_user(Username) ->
               Acc ++ mnesia:dirty_match_object(
                        Tab,
                        #tracked_connection{username = Username, _ = '_'})
-      end, [], rabbit_mnesia:cluster_nodes(running)).
+      end, [], rabbit_nodes:all_running()).
 
 -spec count_connections_in(rabbit_types:vhost()) -> non_neg_integer().
 
@@ -380,7 +371,7 @@ count_connections_in(VirtualHost) ->
                                   [VirtualHost, Err, Node]),
                                 Acc
                         end
-                end, 0, rabbit_mnesia:cluster_nodes(running)).
+                end, 0, rabbit_nodes:all_running()).
 
 %% Returns a #tracked_connection from connection_created
 %% event details.
