@@ -1,17 +1,8 @@
-%% The contents of this file are subject to the Mozilla Public License
-%% Version 1.1 (the "License"); you may not use this file except in
-%% compliance with the License. You may obtain a copy of the License
-%% at http://www.mozilla.org/MPL/
+%% This Source Code Form is subject to the terms of the Mozilla Public
+%% License, v. 2.0. If a copy of the MPL was not distributed with this
+%% file, You can obtain one at https://mozilla.org/MPL/2.0/.
 %%
-%% Software distributed under the License is distributed on an "AS IS"
-%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
-%% the License for the specific language governing rights and
-%% limitations under the License.
-%%
-%% The Original Code is RabbitMQ HTTP authentication.
-%%
-%% The Initial Developer of the Original Code is VMware, Inc.
-%% Copyright (c) 2007-2020 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2020 VMware, Inc. or its affiliates.  All rights reserved.
 %%
 
 -module(rabbit_auth_backend_http).
@@ -59,12 +50,19 @@ user_login_authorization(Username, AuthProps) ->
         Else                          -> Else
     end.
 
-check_vhost_access(#auth_user{username = Username, tags = Tags}, VHost, AuthzData = #{peeraddr := PeerAddr}) ->
+check_vhost_access(#auth_user{username = Username, tags = Tags}, VHost, undefined) ->
+    do_check_vhost_access(Username, Tags, VHost, "", undefined);
+check_vhost_access(#auth_user{username = Username, tags = Tags}, VHost,
+                   AuthzData = #{peeraddr := PeerAddr}) when is_map(AuthzData) ->
     AuthzData1 = maps:remove(peeraddr, AuthzData),
-    OptionsParameters = context_as_parameters(AuthzData1),
+    Ip = parse_peeraddr(PeerAddr),
+    do_check_vhost_access(Username, Tags, VHost, Ip, AuthzData1).
+
+do_check_vhost_access(Username, Tags, VHost, Ip, AuthzData) ->
+    OptionsParameters = context_as_parameters(AuthzData),
     bool_req(vhost_path, [{username, Username},
                           {vhost,    VHost},
-                          {ip,       parse_peeraddr(PeerAddr)},
+                          {ip,       Ip},
                           {tags,     join_tags(Tags)}] ++ OptionsParameters).
 
 check_resource_access(#auth_user{username = Username, tags = Tags},
