@@ -1,17 +1,8 @@
-%% The contents of this file are subject to the Mozilla Public License
-%% Version 1.1 (the "License"); you may not use this file except in
-%% compliance with the License. You may obtain a copy of the License
-%% at https://www.mozilla.org/MPL/
+%% This Source Code Form is subject to the terms of the Mozilla Public
+%% License, v. 2.0. If a copy of the MPL was not distributed with this
+%% file, You can obtain one at https://mozilla.org/MPL/2.0/.
 %%
-%% Software distributed under the License is distributed on an "AS IS"
-%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
-%% the License for the specific language governing rights and
-%% limitations under the License.
-%%
-%% The Original Code is RabbitMQ HTTP authentication.
-%%
-%% The Initial Developer of the Original Code is VMware, Inc.
-%% Copyright (c) 2007-2020 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2020 VMware, Inc. or its affiliates.  All rights reserved.
 %%
 
 -module(rabbit_auth_backend_oauth2).
@@ -208,11 +199,8 @@ post_process_payload_complex_claim(Payload) ->
     case AdditionalScopes of
         [] -> Payload;
         _  ->
-            TokenScopes = maps:get(<<"scope">>, Payload),
-            case TokenScopes of
-                [] -> maps:put(<<"scope">>, AdditionalScopes, Payload);
-                _  -> maps:put(<<"scope">>, AdditionalScopes ++ TokenScopes, Payload)
-            end
+            ExistingScopes = maps:get(<<"scope">>, Payload, []),
+            maps:put(<<"scope">>, AdditionalScopes ++ ExistingScopes, Payload)
         end.
 
 %% keycloak token format: https://github.com/rabbitmq/rabbitmq-auth-backend-oauth2/issues/36
@@ -221,15 +209,8 @@ post_process_payload_keycloak(#{<<"authorization">> := Authorization} = Payload)
         undefined   -> [];
         Permissions -> extract_scopes_from_keycloak_permissions([], Permissions)
     end,
-    case AdditionalScopes of
-        [] -> Payload;
-        _  ->
-            TokenScopes = maps:get(<<"scope">>, Payload),
-            case TokenScopes of
-                [] -> maps:put(<<"scope">>, AdditionalScopes, Payload);
-                _  -> maps:put(<<"scope">>, AdditionalScopes ++ TokenScopes, Payload)
-            end
-    end.
+    ExistingScopes = maps:get(<<"scope">>, Payload),
+    maps:put(<<"scope">>, AdditionalScopes ++ ExistingScopes, Payload).
 
 extract_scopes_from_keycloak_permissions(Acc, []) ->
     Acc;
