@@ -1,17 +1,8 @@
-## The contents of this file are subject to the Mozilla Public License
-## Version 1.1 (the "License"); you may not use this file except in
-## compliance with the License. You may obtain a copy of the License
-## at https://www.mozilla.org/MPL/
+## This Source Code Form is subject to the terms of the Mozilla Public
+## License, v. 2.0. If a copy of the MPL was not distributed with this
+## file, You can obtain one at https://mozilla.org/MPL/2.0/.
 ##
-## Software distributed under the License is distributed on an "AS IS"
-## basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
-## the License for the specific language governing rights and
-## limitations under the License.
-##
-## The Original Code is RabbitMQ.
-##
-## The Initial Developer of the Original Code is Pivotal Software, Inc.
-## Copyright (c) 2016-2019 Pivotal Software, Inc.  All rights reserved.
+## Copyright (c) 2016-2020 VMware, Inc. or its affiliates.  All rights reserved.
 
 defmodule RabbitMQ.CLI.Core.Config do
 
@@ -42,13 +33,13 @@ defmodule RabbitMQ.CLI.Core.Config do
   def normalise(:node, nil), do: nil
 
   def normalise(:node, node) when not is_atom(node) do
-    Rabbitmq.Atom.Coerce.to_atom(node)
+    RabbitMQ.CLI.Core.DataCoercion.to_atom(node)
   end
 
   def normalise(:erlang_cookie, nil), do: nil
 
   def normalise(:erlang_cookie, c) when not is_atom(c) do
-    Rabbitmq.Atom.Coerce.to_atom(c)
+    RabbitMQ.CLI.Core.DataCoercion.to_atom(c)
   end
 
   def normalise(:longnames, true), do: :longnames
