@@ -1,17 +1,8 @@
-## The contents of this file are subject to the Mozilla Public License
-## Version 1.1 (the "License"); you may not use this file except in
-## compliance with the License. You may obtain a copy of the License
-## at https://www.mozilla.org/MPL/
+## This Source Code Form is subject to the terms of the Mozilla Public
+## License, v. 2.0. If a copy of the MPL was not distributed with this
+## file, You can obtain one at https://mozilla.org/MPL/2.0/.
 ##
-## Software distributed under the License is distributed on an "AS IS"
-## basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
-## the License for the specific language governing rights and
-## limitations under the License.
-##
-## The Original Code is RabbitMQ.
-##
-## The Initial Developer of the Original Code is GoPivotal, Inc.
-## Copyright (c) 2007-2020 Pivotal Software, Inc.  All rights reserved.
+## Copyright (c) 2007-2020 VMware, Inc. or its affiliates.  All rights reserved.
 
 defmodule RabbitMQ.CLI.Ctl.Commands.ExportDefinitionsCommand do
   alias RabbitMQ.CLI.Core.{DocGuide, ExitCodes, Helpers}
@@ -129,7 +120,15 @@ defmodule RabbitMQ.CLI.Ctl.Commands.ExportDefinitionsCommand do
   # Implementation
   #
 
-  defp serialise(map, "json") do
+  defp serialise(raw_map, "json") do
+    # make sure all runtime parameter values are maps, otherwise
+    # they will end up being a list of pairs (a keyword list/proplist)
+    # in the resulting JSON document
+    map = Map.update!(raw_map, :parameters, fn(params) ->
+      Enum.map(params, fn(param) ->
+        Map.update!(param, "value", &:rabbit_data_coercion.to_map/1)
+      end)
+    end)
     {:ok, json} = JSON.encode(map)
     json
   end
