@@ -1,22 +1,13 @@
-## The contents of this file are subject to the Mozilla Public License
-## Version 1.1 (the "License"); you may not use this file except in
-## compliance with the License. You may obtain a copy of the License
-## at https://www.mozilla.org/MPL/
+## This Source Code Form is subject to the terms of the Mozilla Public
+## License, v. 2.0. If a copy of the MPL was not distributed with this
+## file, You can obtain one at https://mozilla.org/MPL/2.0/.
 ##
-## Software distributed under the License is distributed on an "AS IS"
-## basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
-## the License for the specific language governing rights and
-## limitations under the License.
-##
-## The Original Code is RabbitMQ.
-##
-## The Initial Developer of the Original Code is GoPivotal, Inc.
-## Copyright (c) 2007-2020 Pivotal Software, Inc.  All rights reserved.
+## Copyright (c) 2007-2020 VMware, Inc. or its affiliates.  All rights reserved.
 
 defmodule RabbitMQ.CLI.Ctl.Commands.StatusCommand do
   alias RabbitMQ.CLI.Core.DocGuide
   alias RabbitMQ.CLI.InformationUnit, as: IU
-  import RabbitMQ.CLI.Core.{Alarms, ANSI, Listeners, Memory, Platform}
+  import RabbitMQ.CLI.Core.{Alarms, ANSI, DataCoercion, Listeners, Memory, Platform}
 
   @behaviour RabbitMQ.CLI.CommandBehaviour
 
@@ -95,6 +86,7 @@ defmodule RabbitMQ.CLI.Ctl.Commands.StatusCommand do
       "OS: #{m[:os]}",
       # TODO: format
       "Uptime (seconds): #{m[:uptime]}",
+      "Is under maintenance?: #{m[:is_under_maintenance]}"
     ] ++
     product_name_section ++
     product_version_section ++
@@ -135,6 +127,8 @@ defmodule RabbitMQ.CLI.Ctl.Commands.StatusCommand do
          end
 
     breakdown = compute_relative_values(m[:memory])
+    memory_calculation_strategy = to_atom(m[:vm_memory_calculation_strategy])
+    total_memory = get_in(m[:memory], [:total, memory_calculation_strategy])
 
     readable_watermark_setting = case m[:vm_memory_high_watermark_setting] do
       %{:relative => val} -> "#{val} of available memory"
@@ -143,8 +137,9 @@ defmodule RabbitMQ.CLI.Ctl.Commands.StatusCommand do
     end
     memory_section = [
       "\n#{bright("Memory")}\n",
-      "Calculation strategy: #{m[:vm_memory_calculation_strategy]}",
-      "Memory high watermark setting: #{readable_watermark_setting}, computed to: #{IU.convert(m[:vm_memory_high_watermark_limit], unit)} #{unit}"
+      "Total memory used: #{IU.convert(total_memory, unit)} #{unit}",
+      "Calculation strategy: #{memory_calculation_strategy}",
+      "Memory high watermark setting: #{readable_watermark_setting}, computed to: #{IU.convert(m[:vm_memory_high_watermark_limit], unit)} #{unit}\n"
     ] ++ Enum.map(breakdown, fn({category, val}) -> "#{category}: #{IU.convert(val[:bytes], unit)} #{unit} (#{val[:percentage]} %)" end)
 
     file_descriptors = [
@@ -217,6 +212,7 @@ defmodule RabbitMQ.CLI.Ctl.Commands.StatusCommand do
       rabbitmq_version: Keyword.get(result, :rabbitmq_version) |> to_string,
       erlang_version: Keyword.get(result, :erlang_version) |> to_string |> String.trim_trailing,
       uptime: Keyword.get(result, :uptime),
+      is_under_maintenance: Keyword.get(result, :is_under_maintenance, false),
       processes: Enum.into(Keyword.get(result, :processes), %{}),
       run_queue: Keyword.get(result, :run_queue),
       net_ticktime: net_ticktime(result),
