@@ -1,17 +1,8 @@
-## The contents of this file are subject to the Mozilla Public License
-## Version 1.1 (the "License"); you may not use this file except in
-## compliance with the License. You may obtain a copy of the License
-## at https://www.mozilla.org/MPL/
+## This Source Code Form is subject to the terms of the Mozilla Public
+## License, v. 2.0. If a copy of the MPL was not distributed with this
+## file, You can obtain one at https://mozilla.org/MPL/2.0/.
 ##
-## Software distributed under the License is distributed on an "AS IS"
-## basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
-## the License for the specific language governing rights and
-## limitations under the License.
-##
-## The Original Code is RabbitMQ.
-##
-## The Initial Developer of the Original Code is GoPivotal, Inc.
-## Copyright (c) 2007-2020 Pivotal Software, Inc.  All rights reserved.
+## Copyright (c) 2007-2020 VMware, Inc. or its affiliates.  All rights reserved.
 
 defmodule RabbitMQ.CLI.Queues.Commands.CheckIfNodeIsQuorumCriticalCommand do
   @moduledoc """
@@ -33,26 +24,42 @@ defmodule RabbitMQ.CLI.Queues.Commands.CheckIfNodeIsQuorumCriticalCommand do
   use RabbitMQ.CLI.Core.RequiresRabbitAppRunning
 
   def run([], %{node: node_name, timeout: timeout}) do
-    case :rabbit_misc.rpc_call(node_name, :rabbit_quorum_queue, :list_with_minimum_quorum_for_cli, [], timeout) do
-      [] -> []
-      qs when is_list(qs) -> qs
+    case :rabbit_misc.rpc_call(node_name, :rabbit_nodes, :is_single_node_cluster, [], timeout) do
+      # if target node is the only one in the cluster, the check makes little sense
+      # and false positives can be misleading
+      true  -> {:ok, :single_node_cluster}
+      false ->
+        case :rabbit_misc.rpc_call(node_name, :rabbit_quorum_queue, :list_with_minimum_quorum_for_cli, [], timeout) do
+          [] -> {:ok, []}
+          qs when is_list(qs) -> {:ok, qs}
+          other -> other
+        end
       other -> other
     end
   end
 
-  def output([], %{formatter: "json"}) do
+  def output({:ok, :single_node_cluster}, %{formatter: "json"}) do
+    {:ok, %{
+      "result"  => "ok",
+      "message" => "Target node seems to be the only one in a single node cluster, the check does not apply"
+    }}
+  end
+  def output({:ok, []}, %{formatter: "json"}) do
     {:ok, %{"result" => "ok"}}
   end
-
-  def output([], %{silent: true}) do
+  def output({:ok, :single_node_cluster}, %{silent: true}) do
     {:ok, :check_passed}
   end
-
-  def output([], %{node: node_name}) do
+  def output({:ok, []}, %{silent: true}) do
+    {:ok, :check_passed}
+  end
+  def output({:ok, :single_node_cluster}, %{node: node_name}) do
+    {:ok, "Node #{node_name} seems to be the only one in a single node cluster, the check does not apply"}
+  end
+  def output({:ok, []}, %{node: node_name}) do
     {:ok, "Node #{node_name} reported no quorum queues with minimum quorum"}
   end
-
-  def output(qs, %{node: node_name, formatter: "json"}) when is_list(qs) do
+  def output({:ok, qs}, %{node: node_name, formatter: "json"}) when is_list(qs) do
     {:error, :check_failed,
      %{
        "result" => "error",
@@ -60,12 +67,10 @@ defmodule RabbitMQ.CLI.Queues.Commands.CheckIfNodeIsQuorumCriticalCommand do
        "message" => "Node #{node_name} reported local queues with minimum online quorum"
      }}
   end
-
-  def output(_qs, %{silent: true}) do
+  def output({:ok, qs}, %{silent: true}) when is_list(qs) do
     {:error, :check_failed}
   end
-
-  def output(qs, %{node: node_name}) when is_list(qs) do
+  def output({:ok, qs}, %{node: node_name}) when is_list(qs) do
     lines = queue_lines(qs, node_name)
 
     {:error, :check_failed, Enum.join(lines, line_separator())}
