@@ -33,7 +33,7 @@ function fmt_si_prefix(num0, max0, thousand, allow_fractions) {
     var power = num_power[2];
     var powers = ['', 'k', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y'];
     return (((power != 0 || allow_fractions) && max <= 10) ? num.toFixed(1) :
-            num.toFixed(0)) + powers[power];
+            num.toFixed(0)) + " " + powers[power];
 }
 
 function fmt_boolean(b, unknown) {
@@ -72,7 +72,7 @@ function fmt_timestamp_mini(ts) {
 }
 
 function fmt_time(t, suffix) {
-    if (t == undefined || t == 0) return '';
+    if (typeof t === "undefined") return '';
     return t + suffix;
 }
 
@@ -314,7 +314,8 @@ function fmt_rate_axis(num, max) {
 
 function fmt_bytes(bytes) {
     if (bytes == undefined) return UNKNOWN_REPR;
-    return fmt_si_prefix(bytes, bytes, 1024, false) + 'iB';
+    var prefix = fmt_si_prefix(bytes, bytes, 1024, false);
+    return prefix + (prefix != bytes ? 'iB' : 'B');
 }
 
 function fmt_bytes_axis(num, max) {
