@@ -1,17 +1,8 @@
-%% The contents of this file are subject to the Mozilla Public License
-%% Version 1.1 (the "License"); you may not use this file except in
-%% compliance with the License. You may obtain a copy of the License at
-%% https://www.mozilla.org/MPL/
+%% This Source Code Form is subject to the terms of the Mozilla Public
+%% License, v. 2.0. If a copy of the MPL was not distributed with this
+%% file, You can obtain one at https://mozilla.org/MPL/2.0/.
 %%
-%% Software distributed under the License is distributed on an "AS IS"
-%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
-%% License for the specific language governing rights and limitations
-%% under the License.
-%%
-%% The Original Code is RabbitMQ Management Plugin.
-%%
-%% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2020 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2020 VMware, Inc. or its affiliates.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_util).
@@ -819,7 +810,14 @@ decode(<<"">>) ->
     {ok, #{}};
 decode(Body) ->
     try
-        {ok, rabbit_json:decode(Body)}
+        %% handle double encoded JSON, see rabbitmq/rabbitmq-management#839
+        case rabbit_json:decode(Body) of
+            Val when is_map(Val)    -> {ok, Val};
+            Val when is_list(Val)   -> {ok, maps:from_list(Val)};
+            Bin when is_binary(Bin) -> {error, "invalid payload: the request body JSON-decoded to a string. "
+                                               "Is the input doubly-JSON-encoded?"};
+            _                       -> {error, not_json}
+        end
     catch error:_ -> {error, not_json}
     end.
 
@@ -1085,16 +1083,24 @@ list_login_vhosts_names(User, AuthzData) ->
     [V || V <- rabbit_vhost:list_names(),
           case catch rabbit_access_control:check_vhost_access(User, V, AuthzData, #{}) of
               ok -> true;
-              _  -> false
+              NotOK ->
+                  log_access_control_result(NotOK),
+                  false
           end].
 
 list_login_vhosts(User, AuthzData) ->
     [V || V <- rabbit_vhost:all(),
           case catch rabbit_access_control:check_vhost_access(User, vhost:get_name(V), AuthzData, #{}) of
               ok -> true;
-              _  -> false
+              NotOK ->
+                  log_access_control_result(NotOK),
+                  false
           end].
 
+% rabbitmq/rabbitmq-auth-backend-http#100
+log_access_control_result(NotOK) ->
+    rabbit_log:debug("rabbit_access_control:check_vhost_access result: ~p", [NotOK]).
+
 %% base64:decode throws lots of weird errors. Catch and convert to one
 %% that will cause a bad_request.
 b64decode_or_throw(B64) ->
