@@ -1,18 +1,10 @@
-%% The contents of this file are subject to the Mozilla Public License
-%% Version 1.1 (the "License"); you may not use this file except in
-%% compliance with the License. You may obtain a copy of the License at
-%% https://www.mozilla.org/MPL/
-%%
-%% Software distributed under the License is distributed on an "AS IS"
-%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
-%% License for the specific language governing rights and limitations
-%% under the License.
-%%
-%% The Original Code is RabbitMQ.
+%% This Source Code Form is subject to the terms of the Mozilla Public
+%% License, v. 2.0. If a copy of the MPL was not distributed with this
+%% file, You can obtain one at https://mozilla.org/MPL/2.0/.
 %%
 %% The Initial Developer of the Original Code is AWeber Communications.
 %% Copyright (c) 2015-2016 AWeber Communications
-%% Copyright (c) 2016-2017 Pivotal Software, Inc. All rights reserved.
+%% Copyright (c) 2016-2020 VMware, Inc. or its affiliates. All rights reserved.
 %%
 
 -module(rabbit_peer_discovery_k8s).
@@ -179,23 +171,23 @@ get_address(Address) ->
     maps:get(list_to_binary(get_config_key(k8s_address_type, M)), Address).
 
 
-generate_v1_event(Namespace, Type, Message, Reason) ->
+generate_v1_event(Namespace, Type, Reason, Message) ->
     {ok, HostName} = inet:gethostname(),
     Name =
         io_lib:format(HostName ++ ".~B",[os:system_time(millisecond)]),
     TimeInSeconds = calendar:system_time_to_rfc3339(erlang:system_time(second)),
-    generate_v1_event(Namespace, Name, Type, Message, Reason, TimeInSeconds, HostName).
+    generate_v1_event(Namespace, Name, Type, Reason, Message, TimeInSeconds, HostName).
 
 
-generate_v1_event(Namespace, Name, Type, Message, Reason, Timestamp, HostName) ->
+generate_v1_event(Namespace, Name, Type, Reason, Message, Timestamp, HostName) ->
     #{
       metadata => #{
 		    namespace => rabbit_data_coercion:to_binary(Namespace),
 		    name => rabbit_data_coercion:to_binary(Name)
 		   },
       type => rabbit_data_coercion:to_binary(Type),
-      message => rabbit_data_coercion:to_binary(Message),
       reason => rabbit_data_coercion:to_binary(Reason),
+      message => rabbit_data_coercion:to_binary(Message),
       count => 1,
       lastTimestamp =>  rabbit_data_coercion:to_binary(Timestamp),
       involvedObject => #{
@@ -236,4 +228,4 @@ send_event(Type, Reason, Message) ->
        [{"Authorization", "Bearer " ++ rabbit_data_coercion:to_list(Token1)}],
        [{ssl, [{cacertfile, get_config_key(k8s_cert_path, M)}]}],
         Body
-      ).
+      ).
\ No newline at end of file
