@@ -1,17 +1,8 @@
-%% The contents of this file are subject to the Mozilla Public License
-%% Version 1.1 (the "License"); you may not use this file except in
-%% compliance with the License. You may obtain a copy of the License at
-%% http://www.mozilla.org/MPL/
+%% This Source Code Form is subject to the terms of the Mozilla Public
+%% License, v. 2.0. If a copy of the MPL was not distributed with this
+%% file, You can obtain one at https://mozilla.org/MPL/2.0/.
 %%
-%% Software distributed under the License is distributed on an "AS IS"
-%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
-%% License for the specific language governing rights and limitations
-%% under the License.
-%%
-%% The Original Code is RabbitMQ.
-%%
-%% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2020 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2020 VMware, Inc. or its affiliates.  All rights reserved.
 %%
 -module(prometheus_rabbitmq_core_metrics_collector).
 -export([register/0,
@@ -233,19 +224,34 @@ collect_mf(_Registry, Callback) ->
     ok.
 
 build_info() ->
+    ProductInfo = rabbit:product_info(),
+    #{product_base_version := BaseVersion} = ProductInfo,
     {ok, PrometheusPluginVersion} = application:get_key(rabbitmq_prometheus, vsn),
     {ok, PrometheusClientVersion} = application:get_key(prometheus, vsn),
+    Properties0 = [
+                   {rabbitmq_version, BaseVersion},
+                   {prometheus_plugin_version, PrometheusPluginVersion},
+                   {prometheus_client_version, PrometheusClientVersion},
+                   {erlang_version, rabbit_misc:otp_release()}
+                  ],
+    Properties1 = case ProductInfo of
+                      #{product_version := ProductVersion} ->
+                          [{product_version, ProductVersion} | Properties0];
+                      _ ->
+                          Properties0
+                  end,
+    Properties = case ProductInfo of
+                     #{product_name := ProductName} ->
+                         [{product_name, ProductName} | Properties1];
+                     _ ->
+                         Properties1
+                 end,
     {
         build_info,
         untyped,
         "RabbitMQ & Erlang/OTP version info",
         [{
-            [
-                {rabbitmq_version, rabbit_misc:version()},
-                {prometheus_plugin_version, PrometheusPluginVersion},
-                {prometheus_client_version, PrometheusClientVersion},
-                {erlang_version, rabbit_misc:otp_release()}
-            ],
+            Properties,
             1
         }]
     }.
