@@ -1,17 +1,8 @@
-%% The contents of this file are subject to the Mozilla Public License
-%% Version 1.1 (the "License"); you may not use this file except in
-%% compliance with the License. You may obtain a copy of the License
-%% at https://www.mozilla.org/MPL/
+%% This Source Code Form is subject to the terms of the Mozilla Public
+%% License, v. 2.0. If a copy of the MPL was not distributed with this
+%% file, You can obtain one at https://mozilla.org/MPL/2.0/.
 %%
-%% Software distributed under the License is distributed on an "AS IS"
-%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
-%% the License for the specific language governing rights and
-%% limitations under the License.
-%%
-%% The Original Code is RabbitMQ.
-%%
-%% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2020 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2020 VMware, Inc. or its affiliates.  All rights reserved.
 %%
 
 -module(rabbit_stomp).
@@ -24,7 +15,8 @@
 -export([connection_info_local/1,
          emit_connection_info_local/3,
          emit_connection_info_all/4,
-         list/0]).
+         list/0,
+         close_all_client_connections/1]).
 
 -define(DEFAULT_CONFIGURATION,
         #stomp_configuration{
@@ -36,10 +28,22 @@
 start(normal, []) ->
     Config = parse_configuration(),
     Listeners = parse_listener_configuration(),
-    rabbit_stomp_sup:start_link(Listeners, Config).
+    Result = rabbit_stomp_sup:start_link(Listeners, Config),
+    EMPid = case rabbit_event:start_link() of
+              {ok, Pid}                       -> Pid;
+              {error, {already_started, Pid}} -> Pid
+            end,
+    gen_event:add_handler(EMPid, rabbit_stomp_internal_event_handler, []),
+    Result.
 
-stop(_State) ->
-    ok.
+stop(_) ->
+    rabbit_stomp_sup:stop_listeners().
+
+-spec close_all_client_connections(string() | binary()) -> {'ok', non_neg_integer()}.
+close_all_client_connections(Reason) ->
+     Connections = list(),
+    [rabbit_stomp_reader:close_connection(Pid, Reason) || Pid <- Connections],
+    {ok, length(Connections)}.
 
 emit_connection_info_all(Nodes, Items, Ref, AggregatorPid) ->
     Pids = [spawn_link(Node, rabbit_stomp, emit_connection_info_local,
