@@ -1,27 +1,21 @@
-%% The contents of this file are subject to the Mozilla Public License
-%% Version 1.1 (the "License"); you may not use this file except in
-%% compliance with the License. You may obtain a copy of the License
-%% at https://www.mozilla.org/MPL/
+%% This Source Code Form is subject to the terms of the Mozilla Public
+%% License, v. 2.0. If a copy of the MPL was not distributed with this
+%% file, You can obtain one at https://mozilla.org/MPL/2.0/.
 %%
-%% Software distributed under the License is distributed on an "AS IS"
-%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
-%% the License for the specific language governing rights and
-%% limitations under the License.
-%%
-%% The Original Code is RabbitMQ.
-%%
-%% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2012-2016 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2012-2020 VMware, Inc. or its affiliates.  All rights reserved.
 %%
 
 -module(rabbit_web_mqtt_handler).
 -behaviour(cowboy_websocket).
 
--export([init/2]).
--export([websocket_init/1]).
--export([websocket_handle/2]).
--export([websocket_info/2]).
--export([terminate/3]).
+-export([
+    init/2,
+    websocket_init/1,
+    websocket_handle/2,
+    websocket_info/2,
+    terminate/3
+]).
+-export([close_connection/2]).
 
 -include_lib("amqp_client/include/amqp_client.hrl").
 
@@ -88,6 +82,13 @@ websocket_init(State = #state{conn_name = ConnStr, socket = Sock, peername = Pee
          #state.stats_timer),
      hibernate}.
 
+-spec close_connection(pid(), string()) -> 'ok'.
+close_connection(Pid, Reason) ->
+    rabbit_log_connection:info("Web MQTT: will terminate connection process ~p, reason: ~s",
+                               [Pid, Reason]),
+    sys:terminate(Pid, Reason),
+    ok.
+
 websocket_handle({binary, Data}, State) ->
     handle_data(Data, State);
 %% Silently ignore ping and pong frames.
@@ -113,28 +114,33 @@ websocket_info({#'basic.deliver'{}, #amqp_msg{}, _DeliveryCtx} = Delivery,
         {ok, ProcState} ->
             {ok, State #state { proc_state = ProcState }, hibernate};
         {error, _, _} ->
-            {stop, State}
+            stop(State)
     end;
 websocket_info(#'basic.ack'{} = Ack, State = #state{ proc_state = ProcState0 }) ->
     case rabbit_mqtt_processor:amqp_callback(Ack, ProcState0) of
         {ok, ProcState} ->
             {ok, State #state { proc_state = ProcState }, hibernate};
         {error, _, _} ->
-            {stop, State}
+            stop(State)
     end;
 websocket_info(#'basic.consume_ok'{}, State) ->
     {ok, State, hibernate};
 websocket_info(#'basic.cancel'{}, State) ->
-    {stop, State};
+    stop(State);
 websocket_info({reply, Data}, State) ->
     {reply, {binary, Data}, State, hibernate};
 websocket_info({'EXIT', _, _}, State) ->
-    {stop, State};
+    stop(State);
 websocket_info({'$gen_cast', duplicate_id}, State = #state{ proc_state = ProcState,
                                                                  conn_name = ConnName }) ->
-    rabbit_log_connection:warning("Web MQTT disconnecting duplicate client id ~p (~p)~n",
+    rabbit_log_connection:warning("Web MQTT disconnecting a client with duplicate ID '~s' (~p)~n",
                  [rabbit_mqtt_processor:info(client_id, ProcState), ConnName]),
-    {stop, State};
+    stop(State);
+websocket_info({'$gen_cast', {close_connection, Reason}}, State = #state{ proc_state = ProcState,
+                                                                 conn_name = ConnName }) ->
+    rabbit_log_connection:warning("Web MQTT disconnecting client with ID '~s' (~p), reason: ~s~n",
+                 [rabbit_mqtt_processor:info(client_id, ProcState), ConnName, Reason]),
+    stop(State);
 websocket_info({start_keepalives, Keepalive},
                State = #state{ socket = Sock, keepalive_sup = KeepaliveSup }) ->
     %% Only the client has the responsibility for sending keepalives
@@ -146,9 +152,11 @@ websocket_info({start_keepalives, Keepalive},
     {ok, State #state { keepalive = Heartbeater }, hibernate};
 websocket_info(keepalive_timeout, State = #state{conn_name = ConnStr}) ->
     rabbit_log_connection:error("closing Web MQTT connection ~p (keepalive timeout)~n", [ConnStr]),
-    {stop, State};
+    stop(State);
 websocket_info(emit_stats, State) ->
     {ok, emit_stats(State), hibernate};
+websocket_info({ra_event, _, _}, State) ->
+    {ok, State, hibernate};
 websocket_info(Msg, State) ->
     rabbit_log_connection:info("Web MQTT: unexpected message ~p~n",
                     [Msg]),
@@ -156,20 +164,18 @@ websocket_info(Msg, State) ->
 
 terminate(_, _, #state{ proc_state = undefined }) ->
     ok;
-terminate(_, _, State = #state{ proc_state = ProcState,
-                                conn_name  = ConnName }) ->
-    maybe_emit_stats(State),
-    rabbit_log_connection:info("closing Web MQTT connection ~p (~s)~n", [self(), ConnName]),
-    rabbit_mqtt_processor:send_will(ProcState),
-    rabbit_mqtt_processor:close_connection(ProcState),
+terminate(_, _, State) ->
+    stop_rabbit_mqtt_processor(State),
     ok.
 
 %% Internal.
 
-handle_data(Data, State0) ->
+handle_data(Data, State0 = #state{conn_name = ConnStr}) ->
     case handle_data1(Data, State0) of
         {ok, State1 = #state{state = blocked}, hibernate} ->
             {[{active, false}], State1, hibernate};
+        {error, Error} ->
+            stop_with_framing_error(State0, Error, ConnStr);
         Other ->
             Other
     end.
@@ -195,20 +201,38 @@ handle_data1(Data, State = #state{ parse_state = ParseState,
                 {error, Reason, _} ->
                     rabbit_log_connection:info("MQTT protocol error ~p for connection ~p~n",
                         [Reason, ConnStr]),
-                    {stop, State};
+                    stop(State, 1002, Reason);
                 {error, Error} ->
-                    rabbit_log_connection:error("MQTT detected framing error '~p' for connection ~p~n",
-                        [Error, ConnStr]),
-                    {stop, State};
+                    stop_with_framing_error(State, Error, ConnStr);
                 {stop, _} ->
-                    {stop, State}
+                    stop(State)
             end;
-        {error, Error} ->
-            rabbit_log_connection:error("MQTT detected framing error '~p' for connection ~p~n",
-                [ConnStr, Error]),
-            {stop, State}
+        Other ->
+            Other
     end.
 
+stop(State) ->
+    stop(State, 1000, "MQTT died").
+
+stop(State, CloseCode, Error0) ->
+    stop_rabbit_mqtt_processor(State),
+    Error1 = rabbit_data_coercion:to_binary(Error0),
+    {[{close, CloseCode, Error1}], State}.
+
+stop_with_framing_error(State, Error0, ConnStr) ->
+    Error1 = rabbit_misc:format("~p", [Error0]),
+    rabbit_log_connection:error("MQTT detected framing error '~s' for connection ~p~n",
+                                [Error1, ConnStr]),
+    stop(State, 1007, Error1).
+
+stop_rabbit_mqtt_processor(State = #state{state = running,
+                                          proc_state = ProcState,
+                                          conn_name = ConnName}) ->
+    maybe_emit_stats(State),
+    rabbit_log_connection:info("closing Web MQTT connection ~p (~s)~n", [self(), ConnName]),
+    rabbit_mqtt_processor:send_will(ProcState),
+    rabbit_mqtt_processor:close_connection(ProcState).
+
 handle_credits(State0) ->
     case control_throttle(State0) of
         State = #state{state = running} ->
