@@ -1,16 +1,7 @@
-%% The contents of this file are subject to the Mozilla Public License
-%% Version 1.1 (the "License"); you may not use this file except in
-%% compliance with the License. You may obtain a copy of the License
-%% at https://www.mozilla.org/MPL/
+%% This Source Code Form is subject to the terms of the Mozilla Public
+%% License, v. 2.0. If a copy of the MPL was not distributed with this
+%% file, You can obtain one at https://mozilla.org/MPL/2.0/.
 %%
-%% Software distributed under the License is distributed on an "AS IS"
-%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
-%% the License for the specific language governing rights and
-%% limitations under the License.
-%%
-%% The Original Code is RabbitMQ.
-%%
-%% The Initial Developer of the Original Code is GoPivotal, Inc.
 %% Copyright (c) 2007-2020 VMware, Inc. or its affiliates.  All rights reserved.
 %%
 
@@ -22,11 +13,14 @@
 -include_lib("amqp_client/include/amqp_client.hrl").
 
 %% Websocket.
--export([init/2]).
--export([websocket_init/1]).
--export([websocket_handle/2]).
--export([websocket_info/2]).
--export([terminate/3]).
+-export([
+    init/2,
+    websocket_init/1,
+    websocket_handle/2,
+    websocket_info/2,
+    terminate/3
+]).
+-export([close_connection/2]).
 
 -record(state, {
     frame_type,
@@ -88,6 +82,13 @@ websocket_init(State) ->
                        parse_state    = rabbit_stomp_frame:initial_state()},
            #state.stats_timer)}.
 
+-spec close_connection(pid(), string()) -> 'ok'.
+close_connection(Pid, Reason) ->
+    rabbit_log_connection:info("Web STOMP: will terminate connection process ~p, reason: ~s",
+                               [Pid, Reason]),
+    sys:terminate(Pid, Reason),
+    ok.
+
 init_processor_state(#state{socket=Sock, peername=PeerAddr, auth_hd=AuthHd}) ->
     Self = self(),
     SendFun = fun (_Sync, Data) ->
@@ -210,6 +211,8 @@ websocket_info(Msg, State) ->
                     [Msg]),
     {ok, State}.
 
+terminate(_Reason, _Req, #state{proc_state = undefined}) ->
+    ok;
 terminate(_Reason, _Req, #state{proc_state = ProcState}) ->
     rabbit_stomp_processor:flush_and_die(ProcState),
     ok.
@@ -236,6 +239,7 @@ handle_data(Data, State0) ->
             {[{active, false}], State1};
         {error, Error0} ->
             Error1 = rabbit_misc:format("~p", [Error0]),
+            rabbit_log_connection:error("STOMP detected framing error '~s'~n", [Error1]),
             stop(State0, 1007, Error1);
         Other ->
             Other
@@ -244,7 +248,7 @@ handle_data(Data, State0) ->
 handle_data1(<<>>, State) ->
     {ok, ensure_stats_timer(State)};
 handle_data1(Bytes, State = #state{proc_state  = ProcState,
-                                  parse_state = ParseState}) ->
+                                   parse_state = ParseState}) ->
     case rabbit_stomp_frame:parse(Bytes, ParseState) of
         {more, ParseState1} ->
             {ok, ensure_stats_timer(State#state{ parse_state = ParseState1 })};
@@ -275,11 +279,12 @@ maybe_block(State, _) ->
 stop(State) ->
     stop(State, 1000, "STOMP died").
 
-stop(State = #state{proc_state = ProcState}, CloseCode, Error) ->
+stop(State = #state{proc_state = ProcState}, CloseCode, Error0) ->
     maybe_emit_stats(State),
     ok = file_handle_cache:release(),
     rabbit_stomp_processor:flush_and_die(ProcState),
-    {[{close, CloseCode, Error}], State}.
+    Error1 = rabbit_data_coercion:to_binary(Error0),
+    {[{close, CloseCode, Error1}], State}.
 
 %%----------------------------------------------------------------------------
 
