@@ -1,30 +1,32 @@
-%% The contents of this file are subject to the Mozilla Public License
-%% Version 1.1 (the "License"); you may not use this file except in
-%% compliance with the License. You may obtain a copy of the License
-%% at https://www.mozilla.org/MPL/
+%% This Source Code Form is subject to the terms of the Mozilla Public
+%% License, v. 2.0. If a copy of the MPL was not distributed with this
+%% file, You can obtain one at https://mozilla.org/MPL/2.0/.
 %%
-%% Software distributed under the License is distributed on an "AS IS"
-%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
-%% the License for the specific language governing rights and
-%% limitations under the License.
-%%
-%% The Original Code is RabbitMQ.
-%%
-%% The Initial Developer of the Original Code is GoPivotal, Inc.
 %% Copyright (c) 2007-2020 VMware, Inc. or its affiliates.  All rights reserved.
 %%
 
 -module(rabbit_web_stomp_listener).
 
--export([init/0]).
+-export([
+  init/0,
+  stop/1,
+  list_connections/0,
+  close_all_client_connections/1
+]).
 
 %% for testing purposes
 -export([get_binding_address/1, get_tcp_port/1, get_tcp_conf/2]).
 
 -include_lib("rabbitmq_stomp/include/rabbit_stomp.hrl").
 
+-import(rabbit_misc, [pget/2]).
 
-%% --------------------------------------------------------------------------
+-define(TCP_PROTOCOL, 'http/web-stomp').
+-define(TLS_PROTOCOL, 'https/web-stomp').
+
+%%
+%% API
+%%
 
 -spec init() -> ok.
 init() ->
@@ -48,6 +50,42 @@ init() ->
     end,
     ok.
 
+stop(State) ->
+    rabbit_networking:stop_ranch_listener_of_protocol(?TCP_PROTOCOL),
+    rabbit_networking:stop_ranch_listener_of_protocol(?TLS_PROTOCOL),
+    State.
+
+-spec list_connections() -> [pid()].
+list_connections() ->
+    PlainPids = connection_pids_of_protocol(?TCP_PROTOCOL),
+    TLSPids   = connection_pids_of_protocol(?TLS_PROTOCOL),
+    
+    PlainPids ++ TLSPids.
+
+-spec close_all_client_connections(string()) -> {'ok', non_neg_integer()}.
+close_all_client_connections(Reason) ->
+    Connections = list_connections(),
+    [rabbit_web_stomp_handler:close_connection(Pid, Reason) || Pid <- Connections],
+    {ok, length(Connections)}.
+    
+
+%%
+%% Implementation
+%%
+
+connection_pids_of_protocol(Protocol) ->
+    case rabbit_networking:ranch_ref_of_protocol(Protocol) of
+        undefined   -> [];
+        AcceptorRef ->
+            lists:map(fun cowboy_ws_connection_pid/1, ranch:procs(AcceptorRef, connections))
+    end.
+
+-spec cowboy_ws_connection_pid(pid()) -> pid().
+cowboy_ws_connection_pid(RanchConnPid) ->
+    Children = supervisor:which_children(RanchConnPid),
+    {cowboy_clear, Pid, _, _} = lists:keyfind(cowboy_clear, 1, Children),
+    Pid.
+
 start_tcp_listener(TCPConf0, CowboyOpts0, Routes) ->
   NumTcpAcceptors = case application:get_env(rabbitmq_web_stomp, num_tcp_acceptors) of
       undefined -> get_env(num_acceptors, 10);
@@ -65,11 +103,11 @@ start_tcp_listener(TCPConf0, CowboyOpts0, Routes) ->
                             middlewares => [cowboy_router,
                                             rabbit_web_stomp_middleware,
                                             cowboy_handler]},
-  case ranch:start_listener(web_stomp,
-                           ranch_tcp,
-                           RanchTransportOpts,
-                           rabbit_web_stomp_connection_sup,
-                           CowboyOpts) of
+  case ranch:start_listener(rabbit_networking:ranch_ref(TCPConf),
+                            ranch_tcp,
+                            RanchTransportOpts,
+                            rabbit_web_stomp_connection_sup,
+                            CowboyOpts) of
       {ok, _}                       -> ok;
       {error, {already_started, _}} -> ok;
       {error, ErrTCP}                  ->
@@ -79,7 +117,7 @@ start_tcp_listener(TCPConf0, CowboyOpts0, Routes) ->
               [ErrTCP, TCPConf]),
           throw(ErrTCP)
   end,
-  listener_started('http/web-stomp', TCPConf),
+  listener_started(?TCP_PROTOCOL, TCPConf),
   rabbit_log_connection:info(
       "rabbit_web_stomp: listening for HTTP connections on ~s:~w~n",
       [get_binding_address(TCPConf), Port]).
@@ -103,7 +141,7 @@ start_tls_listener(TLSConf0, CowboyOpts0, Routes) ->
                             middlewares => [cowboy_router,
                                             rabbit_web_stomp_middleware,
                                             cowboy_handler]},
-  case ranch:start_listener(web_stomp_secure,
+  case ranch:start_listener(rabbit_networking:ranch_ref(TLSConf),
                             ranch_ssl,
                             RanchTransportOpts,
                             rabbit_web_stomp_connection_sup,
@@ -117,7 +155,7 @@ start_tls_listener(TLSConf0, CowboyOpts0, Routes) ->
               [ErrTLS, TLSConf]),
           throw(ErrTLS)
   end,
-  listener_started('https/web-stomp', TLSConf),
+  listener_started(?TLS_PROTOCOL, TLSConf),
   rabbit_log_connection:info(
       "rabbit_web_stomp: listening for HTTPS connections on ~s:~w~n",
       [get_binding_address(TLSConf), TLSPort]).
