@@ -187,7 +187,7 @@
 %% Internal exports
 -export([count_tracer/1, rate_tracer/2, formatter/5, format_trace_output/1, format_trace_output/2]).
 
--type matchspec()    :: [{[term()], [term()], [term()]}].
+-type matchspec()    :: [{[term()] | '_', [term()], [term()]}].
 -type shellfun()     :: fun((_) -> term()).
 -type formatterfun() :: fun((_) -> iodata()).
 -type millisecs()    :: non_neg_integer().
@@ -525,6 +525,9 @@ format(TraceMsg) ->
         %% {trace, Pid, call, {M, F, Args}}
         {call, [{M,F,Args}]} ->
             {"~p:~p~s", [M,F,format_args(Args)]};
+        %% {trace, Pid, call, {M, F, Args}, Msg}
+        {call, [{M,F,Args}, Msg]} ->
+            {"~p:~p~s ~s", [M,F,format_args(Args), format_trace_output(Msg)]};
         %% {trace, Pid, return_to, {M, F, Arity}}
         {return_to, [{M,F,Arity}]} ->
             {" '--> ~p:~p/~p", [M,F,Arity]};
